"""
Classifies: CHEBI:16385 organic sulfide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_organic_sulfide(smiles: str):
    """
    Determines if a molecule is an organic sulfide (thioether) with structure R-S-R where R ≠ H.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an organic sulfide, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # Find sulfur atoms
    sulfur_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'S']
    if not sulfur_atoms:
        return False, "No sulfur atoms found"
        
    # Check each sulfur atom
    for sulfur in sulfur_atoms:
        # Get neighbors
        neighbors = sulfur.GetNeighbors()
        
        # Check oxidation state - exclude sulfoxides, sulfones etc
        if sulfur.GetTotalNumHs() + sulfur.GetExplicitValence() > 2:
            continue
            
        # Check number of neighbors
        if len(neighbors) != 2:
            continue
            
        # Check that both neighbors are carbon or other non-hydrogen atoms
        if all(n.GetSymbol() != 'H' for n in neighbors):
            # Check bond types - should be single bonds
            bonds = [mol.GetBondBetweenAtoms(sulfur.GetIdx(), n.GetIdx()) for n in neighbors]
            if all(bond.GetBondType() == Chem.BondType.SINGLE for bond in bonds):
                # Check that neighbors are not part of C=S or similar groups
                valid = True
                for n in neighbors:
                    for bond in n.GetBonds():
                        if bond.GetBondType() != Chem.BondType.SINGLE and \
                           bond.GetOtherAtomIdx(n.GetIdx()) == sulfur.GetIdx():
                            valid = False
                            break
                if valid:
                    neighbor_symbols = [n.GetSymbol() for n in neighbors]
                    return True, f"Contains R-S-R structure where R is {' and '.join(neighbor_symbols)}"
                
    return False, "No sulfur atom with appropriate R-S-R structure found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:16385',
                          'name': 'organic sulfide',
                          'definition': 'Compounds having the structure RSR (R '
                                        '=/= H). Such compounds were once '
                                        'called thioethers.',
                          'parents': ['CHEBI:26822', 'CHEBI:33261']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.5702479338842975 is too low.\n'
               'True positives: '
               "[('Cc1c(O)cccc1C(=O)N[C@@H](CSc1ccccc1)[C@H](O)CN1C[C@H]2CCCC[C@H]2C[C@H]1C(=O)NC(C)(C)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C[C@H](CSC(=O)c1ccccc1)C(=O)N1C[C@H](C[C@H]1C(O)=O)Sc1ccccc1', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=C(C(=CC=C1)C)NC(=O)CSC2=[NH+]C=NC3=C2NC=N3', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('OC([C@H](CSCCNC(C)=O)N)=O', 'Contains R-S-R structure where "
               "R is carbon'), "
               "('CC(C)(C)C(=O)CSC1=NC2=C(CCC2)C(=C1C#N)C3=CC=CS3', 'Contains "
               "R-S-R structure where R is carbon'), ('CSc1c(C)cc(O)cc1C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('NCCCCCCSc1nc2c(nc(N)[nH]c2=O)n1[C@@H]1O[C@@H]2COP(O)(=O)O[C@H]2[C@H]1O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('Nc1ncnc2n([C@@H]3O[C@@H]4COP(O)(=O)O[C@H]4[C@H]3O)c(Sc3ccc(Cl)cc3)nc12', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C1=CC=CC=C1)CCC2C(=O)N(C3=CC=CC=C3)N(C2=O)C4=CC=CC=C4', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CN1C2=CC=CC=C2C(=C1SC3=CC=C(C=C3)Cl)C=O', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('S([C@@H]1O[C@@H]([C@@H](O)[C@H](O)[C@H]1O)CO)C(=NO)CCCCCSC', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CNC(=O)C1=CC=CC=C1SC2=CC3=C(C=C2)C(=NN3)C=CC4=CC=CC=N4', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCOC(=O)CC(=O)CSC1=NC2=C(CCC2)C=C1C#N', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('OC([C@H](CCC(N[C@H](C(NCC(O)=O)=O)CSC)=O)N)=O', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('S(C=1C=C2OC(=O)C(=C2C=CC1)C3=CC=CC=C3)C4=CC=C(O)C=C4', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCNc1nc(NC(C)(C)C)nc(SC)n1', 'Contains R-S-R structure where "
               "R is carbon'), ('CSCCC(=O)C(\\\\O)=C\\\\O', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('ClC=1C(S(=O)(=O)C(C)C)=C(SC1)SC', 'Contains R-S-R structure "
               "where R is carbon'), ('ClC1NC(NCC1C(=O)OCC)SC', 'Contains "
               "R-S-R structure where R is carbon'), ('CSCCO', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CN1C(=NN=N1)SC2=NC=NC3=C2C(=CS3)C4=CC(=CC=C4)C(F)(F)F', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1CCC2=C(C1)SC3=C2C(=NC(=N3)CN4CCOCC4)SC5=CC=CC=C5', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C=1N=CC=CC1C(O)=O)C2=CC=CC=C2', 'Contains R-S-R structure "
               "where R is carbon'), ('S(C1=CC=C(O)C=C1)C=2C3=C(C=CC=C3)NC2', "
               "'Contains R-S-R structure where R is carbon'), "
               "('ClC1=CC=CC(Cl)=C1CSC1=NN=C(S1)C1=NC=CN=C1', 'Contains R-S-R "
               "structure where R is carbon'), ('S(CCCC)C', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CCOP(=S)(OCC)SCSc1ccc(Cl)cc1', 'Contains R-S-R structure "
               "where R is carbon'), "
               "('C1=CNC(=C1)C(=O)CSC2=NN=C(O2)C3=CC=CO3', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CC(C)(C)C1=NN2C(=NN=C2SC1)C3CCCCC3', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('C1=CC=NC(=C1)C=NNC2=NC(=O)C(S2)CC3=CC(=CC=C3)SC(F)F', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CSc1ncnc2n(cnc12)[C@@H]1O[C@H](CO)[C@@H](O)[C@H]1O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCC(=O)N(C)C=C1C(=O)C2=CC=CC=C2S1', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('[H]C(Cl)=C(Cl)SC[C@H](N)C(O)=O', 'Contains R-S-R structure "
               "where R is carbon'), "
               "('S(C=1N=C(NN1)C(F)(F)F)C2=NC=C([N+]([O-])=O)C=C2', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('O=C(O)[C@@H](N(O)O)CCCCSC', 'Contains R-S-R structure where "
               "R is carbon'), ('CC(C)=CCSC[C@H](N)C(O)=O', 'Contains R-S-R "
               "structure where R is carbon'), ('CSC', 'Contains R-S-R "
               "structure where R is carbon'), ('O=C(O)[C@@H](N)CCCCCSC', "
               "'Contains R-S-R structure where R is carbon'), "
               "('N[C@@H](CSc1ccc(Br)cc1)C(O)=O', 'Contains R-S-R structure "
               "where R is carbon'), "
               "('C1=CC=C(C=C1)CSC2=CC=C(O2)C=C3C(=N)N4C(=NC3=O)SC=N4', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C1=CC=CC=C1)CC(OC)=O', 'Contains R-S-R structure where R "
               "is carbon'), ('CSc1ccc(O)cc1', 'Contains R-S-R structure where "
               "R is carbon'), ('CC(C)Sc1ncnc2nc[nH]c12', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('S(C[C@H](NC(=O)CC[C@H](N)C(O)=O)C(=O)NCC(O)=O)C(=NO)CCCCCSC', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC(C)(Sc1ccc(CCN(CCCCC2CCCCC2)C(=O)NC2CCCCC2)cc1)C(O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CN1C(=NN=N1)SC2=NC=NC3=C2C(=CS3)Br', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('COC(=O)CCSCCO[C@@H]1O[C@H](CO)[C@@H](O[C@@H]2O[C@H](CO)[C@H](O[C@H]3O[C@H](CO)[C@H](O)[C@H](O)[C@H]3O)[C@H](O)[C@H]2O)[C@H](O)[C@H]1O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCN(CC)C(=O)CSC1=NC=NC2=C1SN=C2C3=CC=CS3', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('COC(=O)CCSCCO[C@@H]1O[C@H](CO)[C@H](O[C@H]2O[C@H](CO)[C@H](O)[C@H](O)[C@H]2O)[C@H](O)[C@H]1O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC(=O)N[C@@H](CSC1=CC=CC=C1)C(O)=O', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('C=1(C=CC=CC1N)S/C(=C(/C(/C#N)=C(/SC=2C(=CC=CC2)N)\\\\N)\\\\C#N)/N', "
               "'Contains R-S-R structure where R is carbon'), "
               "('O1[C@@H]([C@H]([C@@H]([C@H]([C@@H]1SC(CCCS(=O)C)=NO)O)O)O)CO', "
               "'Contains R-S-R structure where R is carbon'), "
               "('OC(=O)C1CSC(=N1)c1ccccc1O', 'Contains R-S-R structure where "
               "R is carbon'), ('C(CCO)SCCO', 'Contains R-S-R structure where "
               "R is carbon'), ('CCOC(=O)CSC1=NC(=C(C=C1C#N)C(=O)C)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC([C@H](C(NCCC(NCCSCC(O)=O)=O)=O)O)(C)C)(=O)O)(=O)O)[C@H]([C@H]1O)OP(O)(O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=C(NC(=C1C(=O)OC)C)C(=O)CSC2=CC=C(C=C2)Cl', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('C=1C=NC=CC1CCSC=2C=CC(=CC2)OC', 'Contains R-S-R structure "
               "where R is carbon'), ('C(\\\\CCCCCCCSC)=N/O', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CNCC1=CC=CC=C1SC2=C(C=C(C=C2)[N+]#[C-])N', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CSC[C@H](N)C(O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=CC=C(C=C1)C2=NN=C(O2)SCC3=CC=CC=C3F', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('N[C@@H](CCC(=O)N[C@@H](CS[C@H]1[C@@H]([C@H](C(C1)=O)/C=C/[C@H](CCCCC)O)C/C=C\\\\CCCC(=O)O)C(=O)NCC(=O)O)C(=O)O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('NC(CSCC(N)C(O)=O)C(O)=O', 'Contains R-S-R structure where R "
               "is carbon'), ('CC(C)(C)C(=O)NC(C(Cl)(Cl)Cl)SC1=NC=CC=N1', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCN1C(=NN=C1SCC2=C(C=CC=C2Cl)F)C3=CC=CO3', 'Contains R-S-R "
               "structure where R is carbon'), ('CSc1nc(N)nc(N)n1', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('CN(C)CCCSC1=NC(=NC2=C1CCC2)C3=CC=CC=C3', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('NCCCSC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c(N)ncnc12', "
               "'Contains R-S-R structure where R is carbon')]\n"
               'False positives: '
               "[('C[C@@H]([C@H]1CC[C@H]2[C@@H]3CCC4=CC(=O)CC[C@]4(C)[C@H]3CC[C@]12C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=CC=C(C=C1)OCC(=O)NNC(=O)CSC2=NC=C(C(=N2)N)C#N', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C(CCN(C)C)N1C2=C(SC=3C1=NC=CC3)C=CC=C2', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C[C@H](CCCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C[C@H](CCCCCCCCC/C=C/C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C(NC(CCNC(=O)[C@@H](C(COP(OC[C@@H](C(*)=O)N*)(=O)[O-])(C)C)O)=O)CSC(=O)C[C@@H](C/C=C\\\\CCCCCCCCCCCCCC/C=C\\\\CCCCCCCCCCCCCCCCCC)O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C\\\\C(=C\\\\C1=CSC(C)=N1)\\\\C2CC(C(C)(CCCC(C)(C(C(C)C(C(C)(C)C(CC(=N2)O)O)=O)O)O)SCC(/C(=N/CC(=O)O)/O)/N=C(\\\\CCC(C(=O)O)N)/O)O', "
               "'Contains R-S-R structure where R is carbon'), ('C(CCCCSC)#N', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)CO)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C(NC(CCNC(=O)[C@@H](C(COP(O)(=O)O)(C)C)O)=O)CSC(=O)CCCCC', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@@H]1NC(=O)[C@H]([C@@H](NC(=O)[C@@H](NC(=O)[C@H](NC(=O)C(N(C(CC[C@@H](NC([C@H]([C@@H](NC1=O)/C=C/C(=C/[C@@H]([C@@H](OC)CC2=CC=CC=C2)C)/C)C)=O)C(=O)O)=O)C)=C)C)CC(C)C)C(=O)O)C)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(=O)N[C@@H](CC=2C=3C(NC2)=CC=CC3)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C1=C2C(=C3C(=O)C=4C=CC(=C(C4C(C3=C1)=O)O)[C@@H]5O[C@H]([C@@H](N(C)C)CC5)C)C(=O)C[C@@](C2)(O)C)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('ClC1=C(OC)C(OC)=C2N(C)[C@H]3[C@](C2=C1)(O)[C@@H](O)[C@@]4(SC)C(=O)N(C)[C@@](C(N34)=O)(SC)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC(=O)N(O)CCC[C@H](N)C(=O)N[C@@H](CCCN(O)C(C)=O)C(=O)N[C@@H](CCCN(O)C(C)=O)C(=O)N[C@@H](CO)C(=O)N[C@H]([C@H](O)[C@H]1S[C@H]([C@H](O)[C@H]1O)n1ccc(=N)n(C)c1=O)C(O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C1=CC=C(C=C1)C2=C(C3=CC=CC=C3N2)SCCNC(=O)C4=CC5=CC=CC=C5OC4=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CS)C(=O)NCC(O)=O)C', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('CC1=CC=C(S1)C(=O)CSC2=NC3=CC=CC=C3C(=O)N2C4=CC=CC=C4', "
               "'Contains R-S-R structure where R is carbon'), "
               "('OC[C@H]1O[C@@H](SC(Cc2ccc(O)cc2)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C(=O)([C@@H](NC(=O)C)CCSC)N[C@H](C(=O)*)CC1=CC=CC=C1', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)CNC(=O)[C@@H](N)CCC(O)=O)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C(=O)\\\\C=C\\\\C/C=C\\\\CCCCCCCC)CCNC(CCNC(=O)[C@@H](C(COP(OP(OC[C@H]1O[C@@H](N2C3=C(C(=NC=N3)N)N=C2)[C@@H]([C@@H]1OP([O-])([O-])=O)O)(=O)[O-])(=O)[O-])(C)C)O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S([C@@H]([C@@H](O)CCCC(O)=O)/C=C/C=C/C=C\\\\C/C=C\\\\CCCCC)C[C@H](N)C(O[C@H](COC(=O)CCCCCCCCC)CO)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CCSC)C(=O)N[C@@H](C(C)C)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=C(C(=CC=C1)NC(=O)CSC2=C(C(C(=C(N2)C)C(=O)NC3=CC=CC=C3)C4=CC=CO4)C#N)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CCC(O)=O)C(=O)NCC(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=C(O)C=C1)CC(O)=O)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSCC(=O)SC[C@H](NC(=O)CC[C@H](N)C(O)=O)C(=O)NCC(O)=O)C(=O)NCC(O)=O)C(O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CS)C(=O)N[C@@H](CC(=O)N)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C\\\\C=C\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CN1C(=NN=C1SCC#N)CCC2=NC3=CC=CC=C3N2', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CN(C)CCCN1c2ccccc2Sc2ccc(Cl)cc12', 'Contains R-S-R structure "
               "where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CC=1NC=NC1)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C(CCC)CCOC(=O)C)C', 'Contains R-S-R structure where R is "
               "carbon'), "
               "('S1[C@H]([C@H]2NC(=O)[C@@H](NC(=O)CNC(=O)[C@@H](NC(=O)[C@H]3NC(=O)[C@@H](NC(=O)[C@H]([C@H](CC)C)NC([C@H]4[C@@H](SC[C@H](NC([C@H](NC([C@@H](NC([C@@H](NC([C@H](C1)NC(=O)[C@@H]([N+](C)(C)C)CCC(=O)O)=O)C)=O)CO)=O)[C@@H](SC3)C)=O)C(N[C@@H](CNCCCC[C@H](NC2=O)C(=O)O)C(=O)N[C@H](C(=O)NCC(=O)N[C@H](C(=O)N[C@H](C(N4)=O)C(C)C)[C@H](CC)C)CC5=CC=CC=C5)=O)C)=O)C(C)C)[C@@H](O)C(=O)O)[C@H](O)C)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CSCCC(C(=O)O)NC1=NC=NC2=C1C=C(C=C2)Br', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CC([O-])=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H]([C@H](CC)C)C(O)=O)CC(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)CN)C(=O)N[C@@H](CCCCN)C(O)=O)C', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('S(C[C@@H](NC(=O)C)C(O)=O)C/C=C/CO', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H](CC3=CC=CC=C3)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=CC=C(C=C1)CSC2=NN=C(N2CC3=CC=CS3)C4=CC=CC=N4', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('O=C([O-])[C@@H](NO)CCCCCCCSC', 'Contains R-S-R structure "
               "where R is carbon'), "
               "('C(=O)([C@@H](N)CCSC)N[C@H](C(=O)N[C@H](C(=O)O)CC(O)=O)CC1=CNC2=C1C=CC=C2', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=CC2=C(C=C1)N=C(N2)SCC(=O)NC3=C(N(N(C3=O)C4=CC=CC=C4)C)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('O[N+]([O-])=O.Clc1ccc(C(Cn2ccnc2)OCc2ccc(Sc3ccccc3)cc2)c(Cl)c1', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CO)C(=O)N[C@@H](CC=1NC=NC1)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S1C=2C(=NC(C)(C)C(C2CCC3=C4SC5=C(C(=O)C(C)(C)N[C@]5(C)CC4=NC(C3=O)(C)C)C)=O)C[C@@]6(C1=C(C(=O)C(C)(C)N6)C)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](N)C(=O)N1[C@@H](CCC1)C(=O)N[C@@H](CC(C)C)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CC=1NC=NC1)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCN1C(=O)C2=CC=CC=C2N=C1SCC(=O)OC3CCCCC3', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C=Cc1ccc(O)cc1', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CO)C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C(CC)C(OC)=O)C', 'Contains R-S-R structure where R is "
               "carbon'), "
               "('S(C(=O)CCCCCCCCCCCCCCC)CCOP(OCC[N+](C)(C)C)([O-])=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C1(C2=C(CSC3=C1C=CC=C3)C=CC=C2)NC(=O)CCCN4CCN(CC4)C5=CC=C(C=C5)F.C(/C=C\\\\C(O)=O)(O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C(NC(CCNC(=O)[C@@H](C(COP(OC[C@@H](C(*)=O)N*)(=O)[O-])(C)C)O)=O)CSC(=O)C[C@@H](CCC/C=C\\\\CCCCCCCCCCCCCC/C=C\\\\CCCCCCCCCCCCCCCCCC)O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CC=1NC=NC1)C(=O)N[C@@H](C(C)C)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('[Na+].[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)c1c(C)onc1-c1ccccc1)C([O-])=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CCCN=C(N)N)C)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@H]1N(CCC1)C(=O)[C@@H](N)CCC(O)=O)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C1[C@H]2[C@@H]([C@@H]([C@H](O2)N3C4=C(C(=NC=N4)N)N=C3SC5=CC=C(C=C5)Cl)O)OP(=O)(O1)O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('COc1cc(\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)n2cnc3c(N)ncnc23)ccc1O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C(CC(=O)C)C)CC=1OC=CC1', 'Contains R-S-R structure where R "
               "is carbon'), "
               "('S(CCC1NC(=O)C(NC(=O)NC(C(=O)O)C(CC)C)CCCCNC(=O)C(NC(C(N(C(C(NC1=O)CCC2=CC=CC=C2)=O)C)CCC3=CC=CC=C3)=O)CCSC)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=CC=C(C=C1)C2=NN=C(O2)SCC(=O)N3C4=C(C=C(C=C4)OC)C(=CC3(C)C)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CCCN1C=C(/C=C/C(=O)NC2=C(O)CCC2=O)C=C1)C', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('C1CC2=C(C1)C=C(C=C2)NC(=O)CSC3=NC(=CC(=O)N3)O', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('[Zn+2].S1C2N(C(C(=O)[O-])=C(C1)CC(=O)C)C(=O)C2NC(=O)CCCC(N)C(=O)[O-]', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC(=O)CC[C@H](CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)C(C)=C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=CC2=C(C=C1)C=C(C(=O)N2)C=C3C(=O)N(C(=N)S3)C(=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('[H][N+]([H])(CC[N+]([H])([H])Cc1ccccc1)Cc1ccccc1.[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)COc1ccccc1)C([O-])=O.[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)COc1ccccc1)C([O-])=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCOC1=CC=C(C=C1)C=C2C(=O)N(C(=S)S2)CCC(=O)O', 'Contains "
               "R-S-R structure where R is carbon'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC(C)CCC[C@H](C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C1=C(OC)C=C(C2=NC=CC=C2)N=C1/C=N\\\\O)C', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CC(C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)=C1CC[C@H]2[C@@H]3CCC4=CC(=O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(C1=C2C(N3C(=O)C=CC=4C3=C2C=CN4)=CC=C1)C', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('[H][C@]12CC[C@@]3(C)[C@@]([H])(CC[C@@]33CCC(=O)O3)[C@]1([H])[C@@H](CC1=CC(=O)CC[C@]21C)SC(C)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OC[C@H](N-*)C(-*)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CSC[C@@H](O)[C@@H](O)[C@@H](O)C=O', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\C=C\\\\c1ccc2OCOc2c1', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCCC\\\\C=C/CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OC[C@H](N-*)C(-*)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C[C@@H]1CC2(SCC=N2)[C@]2(O)O[C@@H]3C[C@@]4(C)[C@@H](C[C@@H]5O[C@]55[C@@H]4[C@H](O)C(=O)[C@]4(C)[C@H](CC[C@]54O)C4=CC(=O)OC4)C[C@H]3O[C@@H]2O1', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)C)C(=O)N[C@@H](C)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C1C(=O)NC2=C(O1)C=CC(=C2)C(=O)CSC3=NN=C(N3CC4=CC=CC=C4)C5=CC=CS5', "
               "'Contains R-S-R structure where R is carbon'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSCC(=O)c1ccc(cc1)N=[N+]=[N-])C(=O)NCC(O)=O)C(O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('[NH2+]([C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(=O)N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(C[C@H](C([O-])=O)[C@H](CC)C)=O)CCCC[NH3+])=O)[C@@H](C)CC)=O)CCCNC(N)=[NH2+])=O)CCCC[NH3+])=O)C(C)C)=O)CC(C)C)=O)C)=O)CC(C)C)=O)CCCNC(N)=[NH2+])=O)CC(C)C)=O)CCCC[NH3+])=O)CCCC[NH3+])=O)C)=O)CCCC[NH3+])=O)CO)=O)CCCC[NH3+])=O)C(C)C)=O)C(C)C)=O)[C@@H](C)CC)CCCC[NH3+])=O)[C@H](CC)C)=O)CCCC[NH3+])=O)CO)=O)CCCC[NH3+])=O)C(C)O)C(=O)CCCC[C@@H]1[C@]2([C@@](CS1)(NC(N2)=O)[H])[H]', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CS)C(=O)N[C@@H](CC(C)C)C(O)=O)C', "
               "'Contains R-S-R structure where R is carbon'), "
               "('S1C=2OC3=C(C(O)=CC=C3)C(C2[C@](O)(C(=O)OC)[C@@H]([C@@H]1C)O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)[C@@H](CC([O-])=O)Cc1ccccc1', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OC[C@H](N-*)C(-*)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CC1=CC=CC=C1CSCCNC(=S)NC2=CC=CC=C2', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('CC(C)OC(=O)C(C(=O)OC(C)C)=C1SCCS1', 'Contains R-S-R "
               "structure where R is carbon'), "
               "('C(NC(CCNC(=O)[C@@H](C(COP(OC[C@@H](C(*)=O)N*)(=O)[O-])(C)C)O)=O)CSC(CC(N)=O)=O', "
               "'Contains R-S-R structure where R is carbon'), "
               "('CCOC(=O)OC(C)OC(=O)[C@H]1C(S[C@@H]2N1C(=O)[C@H]2NC(=O)[C@@H](C3=CC=CC=C3)N)(C)C', "
               "'Contains R-S-R structure where R is carbon')]\n"
               "False negatives: [('[*]S\\\\C([*])=N/[*]', 'No sulfur atom "
               "with appropriate R-S-R structure found'), "
               "('COC(=O)Nc1nc2cc(ccc2[nH]1)S(=O)(=O)CC(C)O', 'No sulfur atom "
               "with appropriate R-S-R structure found'), "
               "('C1COCCN1C2=NC3=C(C(=N2)NCC4=NC5=CC6=CC=CC=C6C=C5N4)N=CN3C7=CSC=C7', "
               "'No sulfur atom with appropriate R-S-R structure found'), "
               "('OC/C(=C\\\\[C@@H](NC1=NC=NC2=C1NC=N2)C)/C', 'No sulfur atoms "
               "found')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 70,
    'num_false_positives': 100,
    'num_true_negatives': 1732,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.4117647058823529,
    'recall': 0.958904109589041,
    'f1': 0.5761316872427983,
    'accuracy': 0.9459317585301837}


"""
Classifies: CHEBI:138366 bile acids
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_bile_acids(smiles: str):
    """
    Determines if a molecule is a bile acid based on structural characteristics.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a bile acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for steroid core (tetracyclic)
    ring_info = mol.GetRingInfo()
    if len(ring_info.AtomRings()) < 4:
        return False, "Missing tetracyclic steroid core"
        
    # Look for carboxylic acid group or hydroxy groups
    has_carboxyl = False
    has_hydroxy = False
    
    for atom in mol.GetAtoms():
        # Check for carboxylic acid
        if atom.GetSymbol() == 'O':
            neighbors = atom.GetNeighbors()
            if len(neighbors) == 1 and neighbors[0].GetSymbol() == 'C':
                carbon = neighbors[0]
                if len([n for n in carbon.GetNeighbors() if n.GetSymbol() == 'O']) == 2:
                    has_carboxyl = True
                    
        # Check for hydroxy groups
        if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 1:
            has_hydroxy = True
            
    if not (has_carboxyl or has_hydroxy):
        return False, "Missing characteristic acid/hydroxy groups"
        
    # Check for steroid skeleton with correct carbon count
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 20:
        return False, "Too few carbons for bile acid skeleton"
        
    # Check for cyclopentanoperhydrophenanthrene core
    matches = mol.GetSubstructMatches(Chem.MolFromSmarts('[C]1[C][C]2[C][C][C]3[C][C][C]4[C][C][C][C]4[C]3[C][C]2[C]1'))
    if not matches:
        return False, "Missing steroid core structure"
        
    return True, "Contains steroid core with characteristic acid/hydroxy groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138366',
                          'name': 'bile acids',
                          'definition': 'Any member of a group of hydroxy '
                                        'steroids occuring in bile, where they '
                                        'are present as the sodium salts of '
                                        'their amides with glycine or taurine. '
                                        'In mammals bile acids almost '
                                        'invariably have 5beta-configuration, '
                                        'while in lower vertebrates, some bile '
                                        'acids, known as allo-bile acids, have '
                                        '5alpha-configuration.',
                          'parents': [   'CHEBI:25384',
                                         'CHEBI:35350',
                                         'CHEBI:36078']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183512,
    'num_false_negatives': 42,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9997711845015635}


"""
Classifies: CHEBI:24067 fluoroalkane
"""
from rdkit import Chem

def is_fluoroalkane(smiles: str):
    """
    Determines if a molecule is a fluoroalkane (an alkane with at least one hydrogen replaced by fluorine).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a fluoroalkane, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if molecule contains fluorine and carbon
    if not any(atom.GetSymbol() == 'F' for atom in mol.GetAtoms()):
        return False, "No fluorine atoms present"
    if not any(atom.GetSymbol() == 'C' for atom in mol.GetAtoms()):
        return False, "No carbon atoms present"

    # Check if molecule contains only C, H, and F
    allowed_atoms = ['C', 'H', 'F']
    if not all(atom.GetSymbol() in allowed_atoms for atom in mol.GetAtoms()):
        return False, "Molecule contains atoms other than C, H, and F"

    # Check if molecule is an alkane
    rings = mol.GetRingInfo().AtomRings()
    if rings:
        return False, "Molecule contains rings, not an alkane"

    # Check if molecule contains at least one C-F bond
    has_c_f_bond = False
    for bond in mol.GetBonds():
        atom1 = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
        atom2 = mol.GetAtomWithIdx(bond.GetEndAtomIdx())
        if (atom1.GetSymbol() == 'C' and atom2.GetSymbol() == 'F') or (atom1.GetSymbol() == 'F' and atom2.GetSymbol() == 'C'):
            has_c_f_bond = True
            break

    if has_c_f_bond:
        return True, "Molecule is a fluoroalkane"
    else:
        return False, "Molecule does not contain a C-F bond"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24067',
                          'name': 'fluoroalkane',
                          'definition': 'A haloalkane that is an alkane in '
                                        'which at least one hydrogen atom has '
                                        'been replaced by a fluorine atom.',
                          'parents': ['CHEBI:24469', 'CHEBI:37143']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: module 'rdkit.Chem.Descriptors' has no "
               "attribute 'MolFromSmiles'",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 10,
    'num_true_negatives': 183912,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.09090909090909091,
    'recall': 1.0,
    'f1': 0.16666666666666669,
    'accuracy': 0.9999456294210077}


"""
Classifies: CHEBI:141616 hapalindole-type alkaloid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors
from typing import Tuple

def is_hapalindole_type_alkaloid(smiles: str) -> Tuple[bool, str]:
    """
    Determines if a molecule is a hapalindole-type alkaloid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a hapalindole-type alkaloid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of an indole ring
    indole_ring = None
    for ring in mol.GetRingInfo().AtomRings():
        atoms = [mol.GetAtomWithIdx(i) for i in ring]
        if len(ring) == 9 and all(atom.GetSymbol() == 'C' or atom.GetSymbol() == 'N' for atom in atoms):
            indole_ring = ring
            break
    if indole_ring is None:
        return False, "No indole ring found"

    # Check for the presence of an isonitrile group
    isonitrile_atom = None
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and sum(1 for neighbor in atom.GetNeighbors() if neighbor.GetSymbol() == 'N' and neighbor.GetIsotope() == 0 and neighbor.GetTotalNumHs() == 0) == 1:
            isonitrile_atom = atom.GetIdx()
            break
    if isonitrile_atom is None:
        return False, "No isonitrile group found"

    # Check for the carbon-carbon motif (C10-C11) connecting the indole and isonitrile groups
    c10_atom = None
    c11_atom = None
    for bond in mol.GetBonds():
        atom1 = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
        atom2 = mol.GetAtomWithIdx(bond.GetEndAtomIdx())
        if atom1.GetIdx() in indole_ring and atom2.GetIdx() == isonitrile_atom:
            c10_atom = atom1.GetIdx()
            c11_atom = atom2.GetIdx()
        elif atom2.GetIdx() in indole_ring and atom1.GetIdx() == isonitrile_atom:
            c10_atom = atom2.GetIdx()
            c11_atom = atom1.GetIdx()
    if c10_atom is None or c11_atom is None:
        return False, "No carbon-carbon motif connecting indole and isonitrile groups found"

    # Check for the presence of a monoterpene unit
    monoterpene_atoms = set()
    for atom in mol.GetAtoms():
        if atom.GetIdx() != isonitrile_atom and atom.GetIdx() not in indole_ring:
            monoterpene_atoms.add(atom.GetIdx())
    if len(monoterpene_atoms) < 5:
        return False, "No monoterpene unit found"

    return True, "Molecule is a hapalindole-type alkaloid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:141616',
                          'name': 'hapalindole-type alkaloid',
                          'definition': 'Any member of a structurally diverse '
                                        'group of hybrid isoprenoid-indole '
                                        'alkaloids, produced solely by members '
                                        'of the Subsection V cyanobacterial '
                                        'strains. The major classes of '
                                        'hapalindole-type molecules include '
                                        'hapalindoles, fischerindoles, '
                                        'welwitindolinones and ambiguines that '
                                        'share a common molecular feature '
                                        'where an indole and an isonitrile '
                                        'group are connected by a '
                                        'carbon-carbon motif (C10-C11) that is '
                                        'appended with a monoterpene unit. '
                                        'Fusion of the exomethylene carbon '
                                        'C-16 with indole backbones in the '
                                        'tricyclic hapalindoles provides '
                                        'tetracyclic hapalindoles and '
                                        'fischerindoles that, on '
                                        'rearrangement, can also lead to the '
                                        'bridged tetracyclic '
                                        'welwitindolinones. Decoration of '
                                        'tetracyclic hapalindoles with a '
                                        'tert-prenyl group at C-2 of the '
                                        'indole ring results in ambiguines, of '
                                        'which many have a fused pentacyclic '
                                        '6-6-6-5-7 or 6-6-6-5-6 ring system.',
                          'parents': ['CHEBI:38958']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 73,
    'num_true_negatives': 183852,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9995976642780249}


"""
Classifies: CHEBI:23436 cyanogenic glycoside
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Draw import IPythonConsole

def is_cyanogenic_glycoside(smiles: str):
    """
    Determines if a molecule is a cyanogenic glycoside.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a cyanogenic glycoside, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for presence of glycoside (cyclic ether with multiple OH groups)
    # SMARTS pattern for pyranose ring
    pyranose_pattern = "[OX2r6]1[CR4r6][CR4r6][CR4r6][CR4r6][CR4r6]1"
    has_pyranose = len(mol.GetSubstructMatches(Chem.MolFromSmarts(pyranose_pattern))) > 0
    
    # Check for hydroxyl groups
    oh_pattern = "[OH]"
    has_hydroxyls = len(mol.GetSubstructMatches(Chem.MolFromSmarts(oh_pattern))) >= 3
    
    # Check for nitrile (C#N) group
    nitrile_pattern = "C#N" 
    has_nitrile = len(mol.GetSubstructMatches(Chem.MolFromSmarts(nitrile_pattern))) > 0
    
    # Check for O-glycosidic bond
    glycosidic_pattern = "[CR4][OX2][CR4]"
    has_glycosidic = len(mol.GetSubstructMatches(Chem.MolFromSmarts(glycosidic_pattern))) > 0

    if not has_pyranose:
        return False, "Missing pyranose ring structure"
        
    if not has_hydroxyls:
        return False, "Insufficient hydroxyl groups"
        
    if not has_nitrile:
        return False, "Missing nitrile (C#N) group"
        
    if not has_glycosidic:
        return False, "Missing O-glycosidic bond"
        
    # If all required structural features are present
    if has_pyranose and has_hydroxyls and has_nitrile and has_glycosidic:
        return True, "Contains pyranose ring, multiple hydroxyl groups, nitrile group and O-glycosidic bond"
        
    return False, "Does not meet all structural requirements"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23436',
                          'name': 'cyanogenic glycoside',
                          'definition': 'A glycoside in which the aglycone '
                                        'contains a cyanide group. A '
                                        'cyanogenic glycoside can release '
                                        'poisonous hydrogen cyanide if acted '
                                        'upon by some enzyme.',
                          'parents': ['CHEBI:18379', 'CHEBI:24400']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183899,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.999983686963709}


"""
Classifies: CHEBI:26649 serine derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_serine_derivative(smiles: str):
    """
    Determines if a molecule is a serine derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a serine derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Look for serine core structure:
    # NH2-CH(CH2OH)-COOH or NH2-CH(CH2O-)-COO-
    serine_pattern = Chem.MolFromSmarts('[NH2,NH3+][CH]([CH2][OH,O-,OX2])[CX3](=[OX1])[OH,O-,OX2]')
    if not mol.HasSubstructMatch(serine_pattern):
        return False, "Does not contain serine core structure"
        
    # Count number of serine cores
    matches = mol.GetSubstructMatches(serine_pattern)
    if len(matches) > 1:
        return False, "Contains multiple serine residues - likely a peptide"
        
    # Look for modifications at amino group, carboxy group or hydroxy group
    # Get the serine core atoms
    core_atoms = list(matches[0])
    
    # Check for modifications
    modifications = []
    
    # Check amino group modifications
    n_atom = mol.GetAtomWithIdx(core_atoms[0])
    if len(n_atom.GetNeighbors()) > 2:
        modifications.append("N-modified")
        
    # Check carboxy group modifications  
    c_atom = mol.GetAtomWithIdx(core_atoms[3])
    if len(c_atom.GetNeighbors()) > 2:
        modifications.append("C-modified")
        
    # Check hydroxy group modifications
    o_atom = mol.GetAtomWithIdx(core_atoms[2])
    if len(o_atom.GetNeighbors()) > 1:
        modifications.append("O-modified")
        
    if len(modifications) == 0:
        return False, "No modifications found on serine core"
        
    return True, f"Serine derivative with {', '.join(modifications)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26649',
                          'name': 'serine derivative',
                          'definition': 'An amino acid derivative resulting '
                                        'from reaction of serine at the amino '
                                        'group or the carboxy group, or from '
                                        'the replacement of any hydrogen of '
                                        'serine by a heteroatom. The '
                                        'definition normally excludes peptides '
                                        'containing serine residues.',
                          'parents': ['CHEBI:83821']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 65,
    'num_false_positives': 100,
    'num_true_negatives': 158304,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.3939393939393939,
    'recall': 0.9558823529411765,
    'f1': 0.5579399141630902,
    'accuracy': 0.9993500429097885}


"""
Classifies: CHEBI:26188 polyketide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_polyketide(smiles: str):
    """
    Determines if a molecule is a polyketide based on structural patterns.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a polyketide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Check for alternating carbonyl and methylene groups
    carbonyl_pattern = Chem.MolFromSmarts('[#6](=[O])[#6][#6](=[O])')
    if carbonyl_pattern is not None and mol.HasSubstructMatch(carbonyl_pattern):
        has_alternating = True
    else:
        has_alternating = False
        
    # Check for presence of oxygen atoms (common in polyketides)
    num_oxygen = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    if num_oxygen == 0:
        return False, "No oxygen atoms present"
        
    # Check for macrocyclic structure (common in polyketides)
    rings = mol.GetRingInfo()
    has_large_ring = False
    for ring in rings.AtomRings():
        if len(ring) >= 12:  # Typical macrocycle size
            has_large_ring = True
            break
            
    # Check for ketone groups
    ketone_pattern = Chem.MolFromSmarts('[#6](=[O])[#6]')
    if ketone_pattern is not None:
        num_ketones = len(mol.GetSubstructMatches(ketone_pattern))
    else:
        num_ketones = 0
    
    # Check for ester groups
    ester_pattern = Chem.MolFromSmarts('[#6](=[O])[O][#6]')
    if ester_pattern is not None:
        num_esters = len(mol.GetSubstructMatches(ester_pattern))
    else:
        num_esters = 0
    
    # Check for hydroxyl groups
    hydroxyl_pattern = Chem.MolFromSmarts('[O][H]')
    if hydroxyl_pattern is not None:
        num_hydroxyls = len(mol.GetSubstructMatches(hydroxyl_pattern))
    else:
        num_hydroxyls = 0
    
    # Combine evidence
    features = []
    if has_large_ring:
        features.append("macrocyclic structure")
    if has_alternating:
        features.append("alternating carbonyl-methylene groups")
    if num_ketones > 0:
        features.append(f"{num_ketones} ketone groups")
    if num_esters > 0:
        features.append(f"{num_esters} ester groups") 
    if num_hydroxyls > 0:
        features.append(f"{num_hydroxyls} hydroxyl groups")
    
    # Decision criteria
    if (has_alternating or (num_ketones >= 2 and num_hydroxyls >= 1)) and len(features) >= 2:
        return True, f"Polyketide structure containing {', '.join(features)}"
    
    return False, "Does not match polyketide structural patterns"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26188',
                          'name': 'polyketide',
                          'definition': 'Natural and synthetic compounds '
                                        'containing alternating carbonyl and '
                                        'methylene groups '
                                        "('beta-polyketones'), biogenetically "
                                        'derived from repeated condensation of '
                                        'acetyl coenzyme A (via malonyl '
                                        'coenzyme A), and usually the '
                                        'compounds derived from them by '
                                        'further condensations, etc. '
                                        'Considered by many to be synonymous '
                                        'with the less frequently used terms '
                                        'acetogenins and ketides.',
                          'parents': ['CHEBI:36963']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.GetSubstructMatches(Mol, NoneType)\n'
               'did not match C++ signature:\n'
               '    GetSubstructMatches(RDKit::ROMol self, RDKit::MolBundle '
               'query, RDKit::SubstructMatchParameters params)\n'
               '    GetSubstructMatches(RDKit::ROMol self, RDKit::ROMol query, '
               'RDKit::SubstructMatchParameters params)\n'
               '    GetSubstructMatches(RDKit::ROMol self, RDKit::MolBundle '
               'query, bool uniquify=True, bool useChirality=False, bool '
               'useQueryQueryMatches=False, unsigned int maxMatches=1000)\n'
               '    GetSubstructMatches(RDKit::ROMol self, RDKit::ROMol query, '
               'bool uniquify=True, bool useChirality=False, bool '
               'useQueryQueryMatches=False, unsigned int maxMatches=1000)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 14,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 100,
    'num_negatives': None,
    'precision': 1.0,
    'recall': 0.12280701754385964,
    'f1': 0.21875,
    'accuracy': 0.12280701754385964}


"""
Classifies: CHEBI:23677 diazole
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_diazole(smiles: str):
    """
    Determines if a molecule contains a diazole ring (5-membered ring with 2 nitrogens and 3 carbons).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains a diazole ring, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find all 5-membered rings
    ri = mol.GetRingInfo()
    rings = ri.AtomRings()
    
    # Check each 5-membered ring
    for ring in rings:
        if len(ring) != 5:
            continue
            
        # Count nitrogens and carbons in the ring
        n_count = 0
        c_count = 0
        for atom_idx in ring:
            atom = mol.GetAtomWithIdx(atom_idx)
            if atom.GetSymbol() == 'N':
                n_count += 1
            elif atom.GetSymbol() == 'C':
                c_count += 1
                
        # Check if ring has exactly 2 nitrogens and 3 carbons
        if n_count == 2 and c_count == 3:
            # Check if atoms in ring are aromatic
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms):
                return True, "Contains diazole ring (5-membered aromatic ring with 2 N and 3 C atoms)"
                
    return False, "No diazole ring found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23677',
                          'name': 'diazole',
                          'definition': 'An azole that is either one of a pair '
                                        'of heterocyclic organic compounds '
                                        'comprising three carbon atoms and two '
                                        'nitrogen atoms arranged in a ring.',
                          'parents': ['CHEBI:68452']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 103,
    'num_false_positives': 100,
    'num_true_negatives': 2280,
    'num_false_negatives': 12,
    'num_negatives': None,
    'precision': 0.5073891625615764,
    'recall': 0.8956521739130435,
    'f1': 0.6477987421383649,
    'accuracy': 0.9551102204408818}


"""
Classifies: CHEBI:138138 epoxy(hydroxy)icosatrienoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_epoxy_hydroxy_icosatrienoic_acid(smiles: str):
    """
    Determines if a molecule is an epoxy(hydroxy)icosatrienoic acid.

    An epoxy(hydroxy)icosatrienoic acid is a nonclassic icosanoid that consists of any
    icosatrienoic acid carrying epoxy and hydroxy substituents.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an epoxy(hydroxy)icosatrienoic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of epoxy and hydroxy groups
    epoxy_count = sum(1 for atom in mol.GetAtoms() if atom.GetIsEpoxide())
    hydroxy_count = sum(1 for atom in mol.GetAtoms() if atom.GetHybridization() == Chem.HybridizationType.SP3 and atom.GetTotalNumHs() == 1)
    if epoxy_count == 0 or hydroxy_count == 0:
        return False, "Missing epoxy or hydroxy group"

    # Check for the presence of three double bonds
    double_bond_count = sum(1 for bond in mol.GetBonds() if bond.GetBondType() == Chem.BondType.DOUBLE)
    if double_bond_count != 3:
        return False, "Not an icosatrienoic acid (incorrect number of double bonds)"

    # Check for the presence of a carboxyl group
    carboxyl_count = sum(1 for atom in mol.GetAtoms() if atom.GetHybridization() == Chem.HybridizationType.SP2 and atom.GetTotalNumHs() == 0 and atom.GetTotalDegree() == 3)
    if carboxyl_count != 1:
        return False, "Not an icosatrienoic acid (missing or incorrect carboxyl group)"

    # Check for the correct number of carbon atoms
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count != 20:
        return False, "Not an icosatrienoic acid (incorrect number of carbon atoms)"

    return True, "Molecule is an epoxy(hydroxy)icosatrienoic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138138',
                          'name': 'epoxy(hydroxy)icosatrienoic acid',
                          'definition': 'A nonclassic icosanoid that consists '
                                        'of any icosatrienoic acid carrying '
                                        'epoxy and hydroxy substituents.',
                          'parents': [   'CHEBI:140345',
                                         'CHEBI:15904',
                                         'CHEBI:61498',
                                         'CHEBI:61703',
                                         'CHEBI:73155']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "'Atom' object has no attribute 'GetIsEpoxide'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:131903 pyranopyrazole
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetSymmSSSR

def is_pyranopyrazole(smiles: str):
    """
    Determines if a molecule contains a pyranopyrazole scaffold (pyran ring ortho-fused to pyrazole).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains pyranopyrazole, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for pyranopyrazole core
    # Matches a 6-membered oxygen-containing ring (pyran) fused to a 5-membered ring with 2 nitrogens (pyrazole)
    pyranopyrazole_pattern = Chem.MolFromSmarts('[O]1[C]=[C]2[N]=[N][C]([C]2[C]=[C]1)')
    
    if mol.HasSubstructMatch(pyranopyrazole_pattern):
        # Get all rings
        rings = GetSymmSSSR(mol)
        
        # Find the pyran and pyrazole rings
        pyran_ring = None
        pyrazole_ring = None
        
        for ring in rings:
            ring_atoms = list(ring)
            if len(ring_atoms) == 6:  # Potential pyran
                atoms = [mol.GetAtomWithIdx(i) for i in ring_atoms]
                if any(atom.GetSymbol() == 'O' for atom in atoms):
                    pyran_ring = ring
            elif len(ring_atoms) == 5:  # Potential pyrazole
                atoms = [mol.GetAtomWithIdx(i) for i in ring_atoms]
                n_count = sum(1 for atom in atoms if atom.GetSymbol() == 'N')
                if n_count == 2:
                    pyrazole_ring = ring
        
        if pyran_ring and pyrazole_ring:
            # Check if rings share exactly 2 atoms (ortho-fused)
            shared_atoms = set(pyran_ring).intersection(set(pyrazole_ring))
            if len(shared_atoms) == 2:
                # Get substituents
                all_ring_atoms = set(pyran_ring).union(set(pyrazole_ring))
                substituents = []
                
                for atom_idx in all_ring_atoms:
                    atom = mol.GetAtomWithIdx(atom_idx)
                    for neighbor in atom.GetNeighbors():
                        if neighbor.GetIdx() not in all_ring_atoms:
                            substituents.append(neighbor.GetSymbol())
                
                if substituents:
                    return True, f"Pyranopyrazole with substituents: {', '.join(set(substituents))}"
                else:
                    return True, "Unsubstituted pyranopyrazole"
                    
        return False, "Contains pyran and pyrazole rings but not properly fused"
        
    return False, "No pyranopyrazole pattern found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:131903',
                          'name': 'pyranopyrazole',
                          'definition': 'An organic heterobicyclic compound '
                                        'whose skeleton consists of a pyran '
                                        'ring which is ortho-fused to a '
                                        'pyrazole ring.',
                          'parents': ['CHEBI:27171', 'CHEBI:38101']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183903,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999836873185214}


"""
Classifies: CHEBI:149742 N-(fatty acyl)-glycine(1-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_N__fatty_acyl__glycine_1__(smiles: str):
    """
    Determines if a molecule is an N-(fatty acyl)-glycine(1-).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an N-(fatty acyl)-glycine(1-), False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for presence of a carboxylate group
    carboxylate_present = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetFormalCharge() == -1:
            neighbor = atom.GetNeighbors()[0]
            if neighbor.GetSymbol() == 'C' and neighbor.GetFormalCharge() == 0:
                carboxylate_present = True
                break
    if not carboxylate_present:
        return False, "No carboxylate group found"

    # Check for presence of an amide group
    amide_present = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'N' and atom.GetFormalCharge() == 0:
            neighbors = atom.GetNeighbors()
            if len(neighbors) == 2:
                if neighbors[0].GetSymbol() == 'C' and neighbors[1].GetSymbol() == 'C':
                    amide_present = True
                    break
    if not amide_present:
        return False, "No amide group found"

    # Check for the presence of a fatty acyl chain
    fatty_acyl_chain = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and len(atom.GetNeighbors()) == 2:
            neighbors = atom.GetNeighbors()
            if neighbors[0].GetSymbol() == 'C' and neighbors[1].GetSymbol() == 'C':
                fatty_acyl_chain = True
                break
    if not fatty_acyl_chain:
        return False, "No fatty acyl chain found"

    return True, "N-(fatty acyl)-glycine(1-)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:149742',
                          'name': 'N-(fatty acyl)-glycine(1-)',
                          'definition': 'an N-acyl-glycine where the fatty '
                                        'acyl chain is not specified, major '
                                        'species at pH 7.3.',
                          'parents': ['CHEBI:136716', 'CHEBI:57670']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'tuple index out of range',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}